MAX_IDLE_MINUTES = 5
INTERNAL_TAG = '[INTERNAL]:'
GENERIC_ERROR_MESSAGE = "Please message us on our slack channel or leave a note at bugs@spotml.io"
SPOT_STRATEGY_SPOT = 'spot'
SPOT_STRATEGY_ON_DEMAND = 'on-demand'
