SCHEDULED = 'SCHEDULED'
RUNNING = 'RUNNING'
ERROR = 'ERROR'
LAUNCHING_INSTANCE = 'LAUNCHING_INSTANCE'
STOPPED_BY_USER = 'STOPPED_BY_USER'
STOPPED_DUE_TO_IDLE_TIME = 'STOPPED_DUE_TO_IDLE_TIME'
STOPPED_DUE_TO_ERROR = 'STOPPED_DUE_TO_ERROR'
WAITING_FOR_SPOT = 'WAITING_FOR_SPOT'
STOPPED_UNKNOWN_REASON = 'STOPPED_UNKNOWN_REASON'
